<?php
if (!defined('BASEPATH'))
    exit('No direct script access allowed');
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\IOFactory;
class Subscriber extends CI_Controller
{
    public function __construct()
    {
        parent::__construct();
        $this->load->model(array(
            'subscriber_model',
            'domain_model'
        ));
        $this->load->helper(array(
            'form',
            'url',
            'date'
        ));
        $this->load->library('form_validation');
        $this->load->library('session', 'encrypt');
        $this->load->library('template');
        date_default_timezone_set('Asia/Calcutta');
        if (!($this->session->userdata('userid')) or !($this->session->userdata('role'))) {
            redirect(site_url("user/logout"));
        }
        $this->load->library('pagination');
        $config['full_tag_open']   = '<ul class="pagination">';
        $config['full_tag_close']  = '</ul>';
        $config['first_link']      = 'First';
        $config['first_tag_open']  = '<li>';
        $config['first_tag_close'] = '</li>';
        $config['last_link']       = 'Last';
        $config['last_tag_open']   = '<li>';
        $config['last_tag_close']  = '</li>';
        $config['next_link']       = 'Next &gt;';
        $config['next_tag_open']   = '<li>';
        $config['next_tag_close']  = '</li>';
        $config['prev_link']       = '&lt; Prev';
        $config['prev_tag_open']   = '<li>';
        $config['prev_tag_close']  = '</li>';
        $config['cur_tag_open']    = '<li class="active"><a href="#">';
        $config['cur_tag_close']   = '</a></li>';
        $config['num_tag_open']    = '<li>';
        $config['num_tag_close']   = '</li>';
        $this->pagination->initialize($config);
    }
    public function index()
    {
        $result['msg']        = '';
        $config['base_url']   = site_url('subscriber/index');
        $config['total_rows'] = $this->subscriber_model->get_total_subscribers(); //$this->db->count_all_results('subscribers');
        $config['per_page']   = 50;
        $this->pagination->initialize($config);
        $page                  = ($this->uri->segment(3)) ? $this->uri->segment(3) : 0;
        $result["count"]       = $this->uri->segment(3) + 1;
        $result["links"]       = $this->pagination->create_links();
        $result['subscribers'] = $this->subscriber_model->get_subscriber_list($config["per_page"], $page);
        $this->template->title('Subscribers List');
        $this->template->set_layout('default')->build("subscriber_list", $result);
    }
    public function search_subscriber()
    {
        $result['msg'] = '';
        if ($this->input->post()) {
            $this->session->set_userdata('keyword', $this->input->post('keyword'));
        }
        $arr                  = array(
            'keyword' => $this->session->userdata('keyword')
        );
        $config['base_url']   = site_url('subscriber/search_subscriber');
        $config['total_rows'] = $this->subscriber_model->search_subscriber_all($arr); //$this->db->count_all_results('subscribers');        
        $config['per_page']   = 50;
        $this->pagination->initialize($config);
        $page                  = ($this->uri->segment(3)) ? $this->uri->segment(3) : 0;
        $result["count"]       = $this->uri->segment(3) + 1;
        $result["links"]       = $this->pagination->create_links();
        $result['subscribers'] = $this->subscriber_model->search_subscriber($arr, $config["per_page"], $page);
        $result['keyword']     = $arr['keyword'];
        $this->template->title('Subscribers List');
        $this->template->set_layout('default')->build("subscriber_list", $result);
    }
    public function subscriber_add()
    {
        $result['msg'] = '';
        if ($_POST) {
            $this->form_validation->set_rules('domain', 'domain name', "trim|required");
            $this->form_validation->set_rules('name', 'subscriber name', "trim|max_length[50]");
            $this->form_validation->set_rules('status', 'subscriber status', "trim|required");
            $this->form_validation->set_rules('email', 'subscriber email', 'trim|required|min_length[5]|max_length[50]|valid_email');
            if ($this->form_validation->run()) {
                $data = array(
                    'name' => $this->input->post("name"),
                    'email' => $this->input->post("email"),
                    'domain_id' => $this->input->post("domain"),
                    'is_active' => $this->input->post("status"),
                    'created' => date("Y-m-d H:i:s")
                );
                if (!$this->check_email_exist($this->input->post("domain"), $this->input->post("email"))) {
                    if ($this->subscriber_model->insert_subscribers($data)) {
                        $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b>New subscriber inserted successfully</b></span></div>');
                        redirect(site_url("subscriber/subscriber_add"));
                    } else {
                        $result['msg'] = "Data not saved. Try again !";
                    }
                } else {
                    $result['msg'] = $this->input->post("email") . " already added!";
                }
            }
        }
        $result['domains'] = $this->domain_model->get_domain_list();
        $this->template->title('Add Subscriber');
        $this->template->set_layout('default')->build("subscriber_add", $result);
    }
    public function subscriber_edit($subscriber = 0)
    {
        $result['msg'] = '';
        if ($_POST) {
            $this->form_validation->set_rules('domain', 'domain name', "trim|required");
            $this->form_validation->set_rules('name', 'subscriber name', "trim|max_length[50]");
            $this->form_validation->set_rules('status', 'subscriber status', "trim|required");
            $this->form_validation->set_rules('email', 'subscriber email', 'trim|required|min_length[5]|max_length[50]|valid_email|xss_clean');
            if ($this->form_validation->run()) {
                $data = array(
                    'name' => $this->input->post("name"),
                    'email' => $this->input->post("email"),
                    'domain_id' => $this->input->post("domain"),
                    'is_active' => $this->input->post("status"),
                    'updated' => date("Y-m-d H:i:s")
                );
                if (!$this->check_email_exist($this->input->post("domain"), $this->input->post("email"),$subscriber)) {
                    if ($this->subscriber_model->update_subscribers($subscriber, $data)) {
                        $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b>Subscriber updated successfully</b></span></div>');
                        redirect(site_url("subscriber"));
                    } else {
                        $result['msg'] = "Data not updated. Try again !";
                    }
                } else {
                    $result['msg'] = $this->input->post("email") . " already added!";
                }
            }
        }
        $result['subscriber'] = $this->subscriber_model->get_required_subscriber($subscriber);
        $result['domains']    = $this->domain_model->get_domain_list();
        $this->template->title('Edit Subscriber');
        $this->template->set_layout('default')->build("subscriber_add", $result);
    }
    public function subscriber_import()
    {
        $result['msg'] = '';
        if ($_POST) 
        {
            if ($this->input->post("by_gmail")) 
            {
                $this->session->set_userdata(array(
                    "domainid" => $this->input->post("domain")
                ));
                redirect("https://accounts.google.com/o/oauth2/auth?client_id=".GOOGLE_API_CLIENT_ID."&redirect_uri=".GOOGLE_REDIRECT_URL."&scope=https://www.google.com/m8/feeds/&response_type=code");
            } 
            else 
            {
                $this->form_validation->set_rules('domain', 'domain name', "trim|required");
                if ($this->form_validation->run()) 
                {
                    if (!empty($_FILES['subscriber_xls']['name'])) 
                    {
                        $file_type = explode('.', $_FILES['subscriber_xls']['name']);
                        if ($file_type[1] == 'xls' || $file_type[1] == 'csv') 
                        { 
                            $path = 'uploads/subscriber_xls/';
                            //require_once APPPATH . "/third_party/PHPExcel.php";
                            $config['upload_path'] = $path;
                            $config['allowed_types'] = 'xlsx|xls|csv';
                            $config['remove_spaces'] = TRUE;
                            $this->load->library('upload', $config);
                            $this->upload->initialize($config);            
                            if (!$this->upload->do_upload('subscriber_xls')) {
                                $error = array('error' => $this->upload->display_errors());
                            } else {
                                $data = array('upload_data' => $this->upload->data());
                            }
                            if(empty($error))
                            {
                                if (!empty($data['upload_data']['file_name'])) 
                                {
                                    $import_xls_file = $data['upload_data']['file_name'];
                                } 
                                else 
                                {
                                    $import_xls_file = 0;
                                }
                                $inputFileName = $path . $import_xls_file;
                             
                                try 
                                {
                                    //$inputFileType = PHPExcel_IOFactory::identify($inputFileName);
                                    //$objReader = PHPExcel_IOFactory::createReader($inputFileType);
                                    //$objPHPExcel = $objReader->load($inputFileName);
                                    //$allDataInSheet = $objPHPExcel->getActiveSheet()->toArray(null, true, true, true);

                                    $arr_file = explode('.', $_FILES['subscriber_xls']['name']);
                                    $extension = end($arr_file);
                                    if('csv' == $extension){
                                        $reader = new \PhpOffice\PhpSpreadsheet\Reader\Csv();
                                    } else {
                                        if('xls'==strtolower($extension)){
                                            $reader = new \PhpOffice\PhpSpreadsheet\Reader\Xls();
                                        }else{
                                            $reader = new \PhpOffice\PhpSpreadsheet\Reader\Xlsx();
                                        }
                                    }
                                    $spreadsheet = $reader->load($inputFileName);
                                    
                                    $allDataInSheet = $spreadsheet->getActiveSheet()->toArray();
                                    //echo "<pre>";
                                    //print_r($allDataInSheet); die;
                                    $flag = true;
                                    $i=0;
                                    
                                    $sql    = "INSERT INTO subscribers(name, email, domain_id)VALUES ";
                                    $sqlstr = "";
                                    $upload_count = 0;
                                    
                                    foreach ($allDataInSheet as $value) 
                                    {
                                      if($flag){
                                        $flag =false;
                                        continue;
                                      }
                                        //if($i > 0)
                                        //{
                                            $u_first_name = $value[0];
                                            $u_email_id = $value[1];
                                          
                                           if ($this->form_validation->valid_emails($u_email_id)) 
                                           {
                                                if (!$this->check_email_exist($this->input->post("domain"), $u_email_id)) {
                                                    if ($this->form_validation->valid_emails($u_email_id)) {
                                                        $first_name = $u_first_name;
                                                        $sqlstr .= "('" . $first_name . "'";
                                                        $sqlstr .= ",'" . $u_email_id . "'";
                                                        $sqlstr .= ",'" . $this->input->post("domain") . "'),";
                                                        $upload_count++;
                                                    }
                                                }
                                            }
                                        //}
                                      
                                      $i++;
                                    }
                                    //echo $sql . rtrim($sqlstr, ",");die;
                                    if($upload_count > 0){
                                        $this->db->query($sql . rtrim($sqlstr, ","));
                                        $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b> '.$upload_count .' records successfully uploaded.</b></span></div>');
                                    }elseif ($upload_count < 1 && count($allDataInSheet) > 1) {
                                        $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b> '.count($allDataInSheet) .' records getting but they already exists so records not uploaded.</b></span></div>');
                                    }else{
                                        $this->session->set_flashdata('message', '<div align="left" style="color:red;" id="notify"><span><br/><b>Records not uploaded.</b></span></div>');
                                    }
                                    redirect(site_url("subscriber"));                         
                                } 
                                catch (Exception $e) 
                                {
                                   $result['msg'] = "<b align='center' style='padding:5px; color:red;'>".$e->getMessage()."</b>";
                                }

                            } 
                            else 
                            {
                                $result['msg'] = "<b align='center' style='padding:5px; color:red;'>Please Select a valid File.</b>";
                            }
                        } 
                        else 
                        {
                            $result['msg'] = "<b align='center' style='padding:5px; color:red;'>Please Select a File.</b>";
                        }
                    }
                }
            }
         }   
        //$result['msg'] = "";
        $result['domains'] = $this->domain_model->get_domain_list();
        $this->template->title('Add Subscriber');
        $this->template->set_layout('default')->build("subscriber_import", $result);   
    }
    public function import_contact()
    {
        ini_set("memory_limit","512M");
        $afftectedRows = 0;
        $domain_id = $this->session->userdata('domainid');
        if ($domain_id <= 0) {
            $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b>OOPS !! Something went wrong. Please try again.</b></span></div>');
            redirect(site_url("subscriber/subscriber_import"));
        }
        $client_id     = GOOGLE_API_CLIENT_ID;
        $client_secret = GOOGLE_CLIENT_SECRET;
        $redirect_uri  = GOOGLE_REDIRECT_URL;
        $max_results = 900000;
        //$max_results   = GOOGLE_CONTACT_MAX_RESULT;
        $auth_code     = $_GET["code"];
        function curl_file_get_contents($url)
        {

            $curl      = curl_init();
            $userAgent = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)';
            curl_setopt($curl, CURLOPT_URL, $url); //The URL to fetch. This can also be set when initializing a session with curl_init().
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE); //TRUE to return the transfer as a string of the return value of curl_exec() instead of outputting it out directly.
            curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 30); //The number of seconds to wait while trying to connect.    
            curl_setopt($curl, CURLOPT_USERAGENT, $userAgent); //The contents of the "User-Agent: " header to be used in a HTTP request.
            curl_setopt($curl, CURLOPT_FOLLOWLOCATION, TRUE); //To follow any "Location: " header that the server sends as part of the HTTP header.
            curl_setopt($curl, CURLOPT_AUTOREFERER, TRUE); //To automatically set the Referer: field in requests where it follows a Location: redirect.
            curl_setopt($curl, CURLOPT_TIMEOUT, 60); //The maximum number of seconds to allow cURL functions to execute.
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE); //To stop cURL from verifying the peer's certificate.
            $contents = curl_exec($curl);
            curl_close($curl);
            return $contents;
        }
        $fields = array(
            'code' => urlencode($auth_code),
            'client_id' => urlencode($client_id),
            'client_secret' => urlencode($client_secret),
            'redirect_uri' => urlencode($redirect_uri),
            'grant_type' => urlencode('authorization_code')
        );
        $post   = '';
        foreach ($fields as $key => $value) {
            $post .= $key . '=' . $value . '&';
        }
        $post = rtrim($post, '&');
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://accounts.google.com/o/oauth2/token');
        curl_setopt($curl, CURLOPT_POST, 5);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $post);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
        $result = curl_exec($curl);
        curl_close($curl);
        $response    = json_decode($result);
        $accesstoken = $response->access_token;
        /*$url = 'https://www.google.com/m8/feeds/contacts/default/full?max-results='.$max_results.'&oauth_token='.$accesstoken;*/
        
        $url         = 'https://www.google.com/m8/feeds/contacts/default/full?start-index=1&max-results=' . $max_results . '&alt=json&v=3.0&oauth_token=' . $accesstoken;
        
        $xmlresponse = curl_file_get_contents($url);
        if ((strlen(stristr($xmlresponse, 'Authorization required')) > 0) && (strlen(stristr($xmlresponse, 'Error ')) > 0)) //At times you get Authorization error from Google.
            {
            $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b>OOPS !! Something went wrong. Please try again.</b></span></div>');
            redirect(site_url("subscriber/subscriber_import"));
        }
        /*$xml = new SimpleXMLElement($xmlresponse);
        $xml->registerXPathNamespace('gd', 'http://schemas.google.com/g/2005');
        $result = $xml->xpath('//gd:email');*/

        $contacts = json_decode($xmlresponse,true);
        $google_contacts = array();
        if (!empty($contacts['feed']['entry'])) {
            foreach($contacts['feed']['entry'] as $contact) {
               //retrieve Name and email address  
                if (!array_key_exists('gd$email', $contact)){
                    continue;
                }
                else
                {
                    $google_contacts[] = array (
                        'name'=> $contact['title']['$t'],
                        'email' => $contact['gd$email'][0]['address'],
                    );  
                }
                
            }               
        }
        //echo "<pre>";
        //print_r($google_contacts);
        //echo "after filter-----<br>";
        $subscribe_array = array();

        foreach ($google_contacts as $title){
            if (!$this->check_email_exist($domain_id, $title['email'])) {
                $subscribe_array[] = array('email'=>$title['email'],'domain_id'=>$domain_id,'name'=>$title['name']);
                $afftectedRows++;
            }
        }
        //print_r($subscribe_array);die;
        if($afftectedRows > 0){
            $this->db->insert_batch('subscribers', $subscribe_array);
            $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b>'.count($google_contacts).' contacts getting from google and '. $afftectedRows . ' contacts successfully imported.</b></span></div>');
        }elseif (count($google_contacts) > 0 && $afftectedRows < 1){
            $this->session->set_flashdata('message', '<div align="left" style="color:green;" id="notify"><span><br/><b>'.count($google_contacts).' contacts getting from google but they already exists so contacts not imported.</b></span></div>');
        }else{
            $this->session->set_flashdata('message', '<div align="left" style="color:red;" id="notify"><span><br/><b>New contacts not found.</b></span></div>');
        }
        
        redirect(site_url("subscriber"));
    }
    public function check_email_exist($domainid = 0, $email = "" ,$id = "")
    {
        $sql = "SELECT id FROM subscribers where BINARY email='$email' and BINARY domain_id='$domainid'";
        if($id!=''){
            $sql.=" and BINARY id!='$id' ";
        }
        $query = $this->db->query($sql);
        if ($query->num_rows() > 0) {
            return true;
        } else {
            return false;
        }
    }

    public function get_subscriber_count($domain_id)
    {
        $subscribers = $this->subscriber_model->get_subscribers_count($domain_id);
        echo $subscribers;
    }
    /*public function update_as_unsubscriber()
    {
    $arr =array('00basavaraju100@gmail.com',
    '18nilu@gmail.com',
    '211560713@ukzn.ac.za',
    '223684629@qq.com');
    $this->db->select("*");
    $this->db->from("subscribers");
    $this->db->where("is_active",1);
    $this->db->where_in("email",$arr);
    $result = $this->db->get()->result_array();
    
    $i=0;
    foreach($result as $row)
    {
    $this->db->where("email",$row["email"]);
    $this->db->update("subscribers11",array("is_active"=>2));
    $i++;
    }
    echo $i;
    }*/
} // End of the controller 